import React from 'react'
import Route from 'react-router/lib/Route'
import Router from 'react-router/lib/Router'
import IndexRoute from 'react-router/lib/IndexRoute'
import useRouterHistory from 'react-router/lib/useRouterHistory'
import createMemoryHistory from 'history/lib/createMemoryHistory'
import { store } from 'redux-extra'

let currentRoute = "#";
let currentStep = 0;
let stepAction = null;
let pushBackAction = null;
let isOneTimeBackAction = false;

class RouteManager {
    constructor() {
        this.appHistory = useRouterHistory(createMemoryHistory)({ queryKey: false });
        var self = this;
        if (window) {
            window.history.pushState({}, "", "#");
            window.onpopstate = function (ev) {
                var backRoute = null;
                window.history.pushState({}, "", "#");
                var routeSplit = currentRoute.split('/');
                if (routeSplit[0] == "") routeSplit.splice(0, 1);
                var startRouteModel = self.routes.route.childRoutes;
                while (routeSplit.length > 0) {
                    var found = false;
                    for (var i = 0; i < startRouteModel.length; i++) {
                        if (startRouteModel[i].path == routeSplit[0]) {
                            if (routeSplit.length == 1) {
                                found = true;
                                backRoute = startRouteModel[i].backroute;
                                routeSplit.splice(0, 1);
                                break;
                            } else {
                                found = true;
                                routeSplit.splice(0, 1);
                                startRouteModel = startRouteModel[i].childRoutes;
                                break;
                            }
                        }
                    }
                    if (!found) { 
                        routeSplit.splice(0, 1);
                    }
                }
                if (currentStep > 0 && stepAction) {
                    if (pushBackAction) {
                        pushBackAction();
                        if(isOneTimeBackAction)
                            pushBackAction = null;
                    }else{
                        currentStep--;
                        store.dispatch({
                            type: stepAction,
                            payload: currentStep
                        });
                    } 
                } else {

                    if (pushBackAction) {
                        pushBackAction();
                    } else {
                        if (backRoute && backRoute != currentRoute && currentRoute != '#') {
                            store.dispatch({
                                type: TRANSITION,
                                payload: { method: 'replace', arg: backRoute }
                            });
                        }
                    }

                }
            }
        }

        if (document) {
            document.addEventListener('AppBuilderCustomNavigation', function (e) {
                if (e.route) {
                    store.dispatch({
                        type: TRANSITION,
                        payload: { method: 'replace', arg: e.route }
                    });
                }
            }, false);
        }
    }
    getHistory() {
        return this.appHistory;
    }
    generateRoutes( model ) {
        this.routes = model;
        
        return (
            <Router history={this.appHistory} routes={model.route}></Router>
        )
    }  
}

let routeManager = new RouteManager();
export default routeManager;
export const TRANSITION = '@@router/TRANSITION'
export const UPDATE_LOCATION = '@@router/UPDATE_LOCATION'
const SELECT_LOCATION = state => state.routing.location

function transition(method) {
    return arg => { 
            store.dispatch({
                type: TRANSITION, payload: { method, arg }
            }); 
    };
}

export const push = transition('push')
export const replace = transition('replace')
export const go = transition('go')
export const goBack = transition('goBack')
export const goForward = transition('goForward')
export const pushstep = function (step, action) {
    currentStep = step;
    stepAction = action;
    store.dispatch({
        type: action,
        payload: step
    });
}
export const setBackAction = function (action, isOneTime = false) {
    pushBackAction = action;
    isOneTimeBackAction = isOneTime;
}
export const routeActions = { push, replace, go, goBack, goForward, pushstep, setBackAction }

function updateLocation(location) {
    currentRoute = location.pathname;
    currentStep = 0;
    stepAction = null;
    if(isOneTimeBackAction)
        pushBackAction = null;
    return {
        type: UPDATE_LOCATION,
        payload: location
    }
}

const initialState = {
    location: undefined
}

export function routeReducer(state = initialState, { type, payload: location }) {
    if (type !== UPDATE_LOCATION) {
        return state
    }
    return { ...state, location }
}

export function syncHistory(history) {
    let unsubscribeHistory, currentKey, unsubscribeStore
    let connected = false, syncing = false

    function middleware(store) {
        unsubscribeHistory = history.listen(location => {
            currentKey = location.key
            if (syncing) {
                return
            }
            store.dispatch(updateLocation(location))
        })
        connected = true
        return next => action => {
            if (!action || action.type !== TRANSITION || !connected) {
                return next(action)
            }
            const { payload: { method, arg } } = action
            history[method](arg)
        }
    }

    middleware.listenForReplays =
        (store, selectLocationState = SELECT_LOCATION) => {
            const getLocationState = () => selectLocationState(store.getState())
            const initialLocation = getLocationState()

            unsubscribeStore = store.subscribe(() => {
                const location = getLocationState()
                if (!location) {
                    history.transitionTo(initialLocation)
                    return
                }
                if (location.key !== currentKey) {
                    syncing = true
                    history.transitionTo(location)
                    syncing = false
                }
            })
        }

    middleware.unsubscribe = () => {
        unsubscribeHistory()
        if (unsubscribeStore) {
            unsubscribeStore()
        }
        connected = false
    }
    return middleware
}