'use strict';
var webpack = require('webpack')
const path = require('path');
var nodeExternals = require('webpack-node-externals');  

var env = process.env.NODE_ENV
var config = {
  name: 'react-redux-extra',
  entry: {
      main: './src'
  },
  output: {
      path: path.resolve('./lib'),
      filename: 'react-redux-extra.js',
      library: 'react-redux-extra',
      libraryTarget: 'umd'
  },
  resolve: {
      extensions: ['.js','.jsx']
  },
  module: {
    rules: [
        {
            test: /\.(js|jsx)$/,
            exclude: /(node_modules|bower_components)/,  
            use: [{
                loader: 'babel-loader',
                options: {
                    babelrc: false,
                    presets: [["es2015", {"modules":false}],"stage-0","react"],
                    plugins: [require('babel-plugin-transform-decorators-legacy').default]
                }
            }]
        }
    ]
  },
  plugins: [
    new webpack.optimize.OccurrenceOrderPlugin(),
    new webpack.DefinePlugin({
      'process.env': {NODE_ENV: '"production"', TARGET: '"web"'}
    }),
  ],
  externals: [nodeExternals()]
};

if (env === 'production') {
  config.plugins.push(
    new webpack.optimize.UglifyJsPlugin({
      compressor: {
        pure_getters: true,
        unsafe: true,
        unsafe_comps: true,
        warnings: false,
        screw_ie8: false
      },
      mangle: {
        screw_ie8: false
      },
      output: {
        screw_ie8: false
      }
    })
  )
}

module.exports = config